#include <bits/stdc++.h>
#include <random>
using namespace std;

#ifdef EVAL
ifstream in("input.txt");
ofstream out("output.txt");
#else
istream &in(cin);
ostream &out(cout);
#endif

#define NOT_TAKEN -1
#define STONE_DROP_PROB 10
#define CITY_DROP_PROB 5
double SPEED_GAP_WEIGHT = 1;
double STONE_SCORE_WEIGHT = 1;
double PATH_PROB_WEIGHT = 1;
double best_solution = -DBL_MAX;

int N, S, M;
double R;
double v_min, v_max;
vector<int> masses, energies;
vector<vector<int>> city_stones;
vector<vector<int>> graph;
vector<vector<int>> cities_best_roads;
pair<double, double> stats;
double mean, stddev;
vector<double> scores;
pair<double, double> global_scores;
double global_best_score;
double global_worst_score;
double speed_gap;
double knapsack_impact, tsp_impact;

std::random_device myRandomDevice;
unsigned seed = myRandomDevice();
std::default_random_engine myRandomEngine(seed);

int get_rand_int(int min, int max) {
    std::uniform_int_distribution<int> myUnifIntDist(min, max);
    return myUnifIntDist(myRandomEngine);
}

void const print_input(const int N, const int S, const int M, const int C,
                       const double R, const float v_min, const float v_max,
                       const vector<int> &masses, const vector<int> &energies,
                       const vector<vector<int>> &availability,
                       const vector<vector<int>> &graph) {
    cerr << N << " " << S << endl;
    cerr << M << " " << C << " " << R << " " << v_min << " " << v_max << endl;
    for (int j = 0; j < M; j++) {
        cerr << masses[j] << " " << energies[j] << endl;
    }
    for (int i = 0; i < M; i++) {
        int lAi = (int)availability[i].size();
        cerr << lAi << endl;
        for (int j = 0; j < lAi; j++) {
            cerr << availability[i][j] << " ";
        }
        cerr << endl;
    }
    for (int i = 0; i < N; i++) {
        for (int j = 0; j < N; j++) {
            cerr << graph[i][j] << " ";
        }
        cerr << endl;
    }

    return;
}

bool can_pickup_stone(int stone, vector<int> &where_pickup, int C) {
    return where_pickup[stone] == NOT_TAKEN && masses[stone] <= C;
}

int pick_next_city(vector<bool> &visited, int current_city) {
    int best_city = NOT_TAKEN;
    for (int i = 0; i < N && best_city == NOT_TAKEN; i++) {
        int target = cities_best_roads[current_city][i];
        if (!visited[target] && current_city != target) {
            if (get_rand_int(0, 100) > CITY_DROP_PROB) {
                best_city = target;
            }
        }
    }
    if (best_city == NOT_TAKEN) {
        for (int i = 0; i < N && best_city == NOT_TAKEN; i++) {
            int target = cities_best_roads[current_city][i];
            if (!visited[target] && current_city != target) {
                best_city = target;
            }
        }
    }
    return best_city;
}

int pick_best_stone(int current_city, vector<int> &where_pickup,
                    double current_speed, int C, vector<int> &path) {
    int best_stone = NOT_TAKEN;
    // city_stones[current_city].size() << " stones" << endl;
    for (int i = 0;
         i < city_stones[current_city].size() && best_stone == NOT_TAKEN; i++) {
        int stone = city_stones[current_city][i];
        if (can_pickup_stone(stone, where_pickup, C) &&
            get_rand_int(0, 100) > STONE_DROP_PROB) {
            best_stone = stone;
        }
    }

    if (best_stone != NOT_TAKEN) {
        if (speed_gap == 0 || current_speed == v_min || tsp_impact == 0) {
            return best_stone;
        }
        // this prob_offset makes it hard to pick a stone early in the path
        double path_prob = double(path.size()) / N;
        path_prob *= path_prob * path_prob * path_prob;
        // this makes the probability of not picking a stone correlated to the
        // gap between v_max and v_min if you don't lose too much speed, it's
        // fine to pick up stones
        double score_prob = scores[best_stone];
        score_prob = (score_prob - global_worst_score) /
                     (global_best_score - global_worst_score);
        score_prob *= score_prob * score_prob;
        double keep_prob =
            100 *
            (((PATH_PROB_WEIGHT * path_prob) + (SPEED_GAP_WEIGHT * speed_gap) +
              (STONE_SCORE_WEIGHT * score_prob)) /
             (SPEED_GAP_WEIGHT + STONE_SCORE_WEIGHT + PATH_PROB_WEIGHT));

        // cerr << "========================================" << endl;
        // cerr << "PATH: " << path.size() << endl;
        // cerr << "keep_prob: " << keep_prob << endl;
        // cerr << "PATH_PROB_WEIGHT * path_prob" << PATH_PROB_WEIGHT * path_prob << endl; 
        // cerr << "SPEED_GAP_WEIGHT * speed_gap" << SPEED_GAP_WEIGHT * speed_gap << endl; 
        // cerr << "STONE_SCORE_WEIGHT * score_prob" << STONE_SCORE_WEIGHT * score_prob << endl;
        // cerr << "SPEED_GAP_WEIGHT + STONE_SCORE_WEIGHT + PATH_PROB_WEIGHT: " << SPEED_GAP_WEIGHT + STONE_SCORE_WEIGHT + PATH_PROB_WEIGHT << endl;
        // double global_stone_power = (best_stone_reward - mean) / stddev;
        if (get_rand_int(0, 100) < keep_prob) {
            return best_stone;
        }
    }
    return NOT_TAKEN;
}

void print_solution(double final_energy, double gauntlet_energy,
                    double time_elapsed, vector<int> &where_pickup,
                    vector<int> &path) {

    out << scientific << setprecision(10) << final_energy << " ";
    out << scientific << setprecision(10) << gauntlet_energy << " ";
    out << scientific << setprecision(10) << time_elapsed << endl;

    for (int i = 0; i < where_pickup.size(); i++) {
        out << where_pickup[i];
        if (i != where_pickup.size() - 1) {
            out << " ";
        } else {
            out << endl;
        }
    }

    for (int i = 0; i < path.size(); i++) {
        out << path[i];
        if (i != path.size() - 1) {
            out << " ";
        } else {
            out << endl;
        }
    }

    out << "***" << endl;
}

void solve(int C, int pickup_from_city) {

    double total_energy = 0, gauntlet_energy = 0, time_elapsed = 0;
    vector<int> where_pickup(M, NOT_TAKEN);
    vector<int> gauntlet_stones;
    vector<int> path;
    int max_carrying_capacity = C;

    vector<bool> visited(N, false);
    double current_mass = 0.0;
    double bounds =
        (double)max_carrying_capacity == 0
            ? 0
            : ((double)(v_max - v_min)) / ((double)max_carrying_capacity);

    int current_city = S;
    int next_city = S;
    double current_speed = v_max;
    path.push_back(S);
    while (next_city != NOT_TAKEN) {
        visited[current_city] = true;
        if(path.size() > pickup_from_city){
            int stone =
                pick_best_stone(current_city, where_pickup, current_speed, C, path);
            if (stone != NOT_TAKEN) {
                current_mass += masses[stone];
                C -= masses[stone];
                gauntlet_energy += energies[stone];
                gauntlet_stones.push_back(stone);
                where_pickup[stone] = current_city;
            }
        }

        // find the next city
        next_city = pick_next_city(visited, current_city);
        current_speed = v_max - (double)(current_mass * bounds);
        if (next_city != NOT_TAKEN) {
            time_elapsed +=
                ((double)graph[current_city][next_city]) / current_speed;
            path.push_back(next_city);
            current_city = next_city;
        }
    }
    path.push_back(S);
    time_elapsed += ((double)(graph[current_city][S])) / current_speed;

    double final_energy = gauntlet_energy - (time_elapsed * R);

    if (final_energy > best_solution) {
        print_solution(final_energy, gauntlet_energy, time_elapsed,
                       where_pickup, path);
        best_solution = final_energy;
    }
}

pair<double, double> get_stats() {
    double total_sum = 0;
    double variance_sum = 0;
    for (int i = 0; i < masses.size(); i++) {
        assert(masses[i] != 0);
        total_sum += ((double)energies[i]) / ((double)masses[i]);
    }

    double mean = total_sum / masses.size();

    for (int i = 0; i < masses.size(); i++) {
        double xi = ((double)energies[i]) / ((double)masses[i]);
        variance_sum += (xi - mean) * (xi - mean);
    }

    double stddev = sqrt(variance_sum / masses.size());
    return {mean, stddev};
}

pair<double, double> get_global_score() {
    double global_best_stone = -DBL_MAX;
    double global_worst_stone = DBL_MAX;
    double score;
    for (int i = 0; i < masses.size(); i++) {
        score = ((double)energies[i]) / ((double)masses[i]);
        score = (score - mean) / stddev;
        global_best_stone = max(global_best_stone, score);
        global_worst_stone = min(global_worst_stone, score);
    }
    return {global_best_stone, global_worst_stone};
}

double estimate_best_knapsack(int C) {
    vector<int> where_pickup(M, NOT_TAKEN);
    int max_carrying_capacity = C;
    double sum = 0;
    int current_city = S;
    int next_city = S;
    double current_speed = v_max;
    double current_mass = 0.0;
    double bounds =
        (double)max_carrying_capacity == 0
            ? 0
            : ((double)(v_max - v_min)) / ((double)max_carrying_capacity);
    vector<bool> visited(N, false);
    while (next_city != NOT_TAKEN) {
        visited[current_city] = true;
        int best_stone = NOT_TAKEN;
        for (int i = 0;
             i < city_stones[current_city].size() && best_stone == NOT_TAKEN;
             i++) {
            int stone = city_stones[current_city][i];
            if (can_pickup_stone(stone, where_pickup, C)) {
                best_stone = stone;
            }
        }
        if (best_stone != NOT_TAKEN) {
            current_mass += masses[best_stone];
            C -= masses[best_stone];
            sum += energies[best_stone];
            where_pickup[best_stone] = current_city;
        }

        // find the next city
        next_city = pick_next_city(visited, current_city);
        current_speed = v_max - (double)(current_mass * bounds);
        if (next_city != NOT_TAKEN) {
            current_city = next_city;
        }
    }
    return sum;
}

double estimate_best_tsp() {
    double time_elapsed = 0;
    for (int i = 0; i < N; i++) {
        time_elapsed += graph[i][cities_best_roads[i][0]];
        // cerr << graph[i][cities_best_roads[i][0]] << " " <<
        // graph[i][cities_best_roads[i][1]] << endl;
    }
    return time_elapsed;
}

int pick_random_ubound_next_city(vector<bool> &visited, int N,
                                 int current_city) {
    // for now, pick the HEAVIEST road
    int best_city = NOT_TAKEN;
    vector<int> candidates;
    for (int i = 0; i < N; i++) {
        if (!visited[i] && current_city != i) {
            candidates.push_back(i);
        }
    }
    if (candidates.size() > 0) {
        best_city = candidates[get_rand_int(0, candidates.size() - 1)];
    }
    return best_city;
}

int pick_heaviest_stone(int current_city, vector<int> &where_pickup, int C) {
    int best_stone = NOT_TAKEN;
    int heaviest_stone = INT_MIN;
    // cerr << "city " << current_city << " has " <<
    // city_stones[current_city].size() << " stones" << endl;
    for (int stone : city_stones[current_city]) {
        // cerr << "considering stone " << stone << " ";
        if (can_pickup_stone(stone, where_pickup, C)) {
            // cerr << " which can be picked up with reward " << stone_reward <<
            // endl;
            if (masses[stone] > heaviest_stone) {
                heaviest_stone = masses[stone];
                best_stone = stone;
            }
        }
    }
    return best_stone;
}

int pick_random_stone(int current_city, vector<int> &where_pickup, int C) {
    int best_stone = NOT_TAKEN;
    vector<int> candidates;
    for (int stone : city_stones[current_city]) {
        if (can_pickup_stone(stone, where_pickup, C)) {
            candidates.push_back(stone);
        }
    }
    if (candidates.size() > 0) {
        best_stone = candidates[get_rand_int(0, candidates.size() - 1)];
    }
    return best_stone;
}

double estimate_worst_tsp(int C) {
    double total_energy = 0, gauntlet_energy = 0, time_elapsed = 0;
    vector<int> where_pickup(M, NOT_TAKEN);
    vector<int> gauntlet_stones;
    vector<int> path;
    int max_carrying_capacity = C;
    vector<bool> visited(N, false);
    double current_mass = 0.0;
    double bounds =
        (double)max_carrying_capacity == 0
            ? 0
            : ((double)(v_max - v_min)) / ((double)max_carrying_capacity);
    int current_city = S;
    int next_city = S;
    double current_speed = v_max;
    path.push_back(S);

    while (next_city != NOT_TAKEN) {
        visited[current_city] = true;
        int stone = pick_random_stone(current_city, where_pickup, C);
        if (stone != NOT_TAKEN) {
            current_mass += masses[stone];
            C -= masses[stone];
            gauntlet_energy += energies[stone];
            gauntlet_stones.push_back(stone);
            where_pickup[stone] = current_city;
        }

        // find the next city
        next_city = pick_random_ubound_next_city(visited, N, current_city);
        current_speed = v_max - (double)(current_mass * bounds);
        if (next_city != NOT_TAKEN) {
            time_elapsed +=
                ((double)graph[current_city][next_city]) / current_speed;
            path.push_back(next_city);
            current_city = next_city;
        }
    }
    path.push_back(S);
    time_elapsed += (double)graph[current_city][S] / current_speed;

    return time_elapsed;
}

#include "endgame.h"
int main() {
    ios_base::sync_with_stdio(false);
    in.tie(NULL);

    int C;
    in >> N >> S >> M >> C >> R >> v_min >> v_max;
    masses.resize(M);
    energies.resize(M);
    city_stones.resize(N);
    int tmp_m, tmp_e, tmp_l, tmp_a;
    // cerr << "Reading 1... " << endl;
    for (int i = 0; i < M; i++) {
        in >> tmp_m >> tmp_e;
        assert(tmp_m != 0);
        masses[i] = tmp_m;
        energies[i] = tmp_e;
    }
    // cerr << "Reading 2... " << endl;
    for (int i = 0; i < M; i++) {
        in >> tmp_l;
        for (int j = 0; j < tmp_l; j++) {
            in >> tmp_a;
            city_stones[tmp_a].push_back(i);
        }
    }
    // cerr << "Sorting stones..." << endl;
    double score1, score2;
    for (vector<int> &stones : city_stones) {
        sort(stones.begin(), stones.end(), [&](int stone1, int stone2) {
            score1 = ((double)energies[stone1]) / ((double)masses[stone1]);
            score2 = ((double)energies[stone2]) / ((double)masses[stone2]);
            return score1 > score2;
        });
    }

    // cerr << "Reading graph..." << endl;
    graph.resize(N);
    for (int i = 0; i < N; i++)
        graph[i].resize(N, INT_MAX);

    for (int i = 1; i < N; i++) {
        for (int j = 0; j < i; j++) {
            int edge_weight;
            in >> edge_weight;
            graph[i][j] = edge_weight;
            graph[j][i] = edge_weight;
        }
    }

    cities_best_roads.resize(N);
    for (int i = 0; i < N; i++) {
        cities_best_roads[i].resize(N);
        for (int j = 0; j < N; j++) {
            cities_best_roads[i][j] = j;
        }
    }

    // cerr << "Sorting roads..." << endl;
    for (int i = 0; i < N; i++) {
        sort(cities_best_roads[i].begin(), cities_best_roads[i].end(),
             [&](int road1, int road2) {
                 return graph[i][road1] < graph[i][road2];
             });
    }

    // cerr << "Sorting roads..." << endl;
    for (int i = 0; i < N; i++) {
        sort(cities_best_roads[i].begin(), cities_best_roads[i].end(),
             [&](int road1, int road2) {
                 return graph[i][road1] < graph[i][road2];
             });
    }

    stats = get_stats();
    mean = stats.first;
    stddev = stats.second;
    scores.resize(M);
    for (int i = 0; i < M; i++) {
        scores[i] = ((double)energies[i]) / ((double)masses[i]);
        scores[i] = (scores[i] - stats.first) / stats.second;
    }

    global_scores = get_global_score();

    global_best_score = global_scores.first;
    global_worst_score = global_scores.second;

    if ((v_max - v_min) == 0) {
        speed_gap = 1;
    } else {
        speed_gap = 1 - ((v_max - v_min) / v_max);
    }

    double best_knapsack = estimate_best_knapsack(C);
    // cerr << "best_knapsack: " << best_knapsack << endl;
    double best_tsp = estimate_best_tsp() / v_max;

    double worst_tsp = estimate_worst_tsp(C);

    // cerr << "best_tsp: " << best_tsp << endl;
    // cerr << "worst_tsp: " << worst_tsp << endl;

    // cerr << "knapsack weight: " << best_knapsack << endl;
    // cerr << "tsp weight: " << R * (worst_tsp - best_tsp) << endl;

    tsp_impact = R * (worst_tsp - best_tsp);
    if (tsp_impact == 0) {
        knapsack_impact = 1;
    } else if (best_knapsack == 0) {
        tsp_impact = 1;
        knapsack_impact = 0;
    } else {
        if (tsp_impact / best_knapsack > 1) {
            knapsack_impact = best_knapsack / tsp_impact;
            tsp_impact = 1 - knapsack_impact;
        } else {
            tsp_impact = tsp_impact / best_knapsack;
            knapsack_impact = 1 - tsp_impact;
        }
    }

    double tot = 5;

    STONE_SCORE_WEIGHT = tot * knapsack_impact;
    tot -= STONE_SCORE_WEIGHT;

    SPEED_GAP_WEIGHT = (speed_gap / v_max) * tot;
    tot -= SPEED_GAP_WEIGHT;

    PATH_PROB_WEIGHT = tot;

    int pickup_from_city = 0;
    if(knapsack_impact < 0.2 && knapsack_impact > 0.1){
        pickup_from_city = N/2;
    } else if(knapsack_impact < 0.1 && knapsack_impact > 0.05) {
        pickup_from_city = N/2 + N/4;
    } else if(knapsack_impact < 0.05 && knapsack_impact > 0.01){
        pickup_from_city = N/2 + N/4 + N/8;
    } else if(knapsack_impact < 0.01){
        pickup_from_city = N/2 + N/4 + N/8 + N/16;
    }

    // cerr << "knapsack_impact: " << knapsack_impact << endl;
    // cerr << "tsp_impact: " << tsp_impact << endl;
    // cerr << "STONE_SCORE_WEIGHT: " << STONE_SCORE_WEIGHT << endl;
    // cerr << "SPEED_GAP_WEIGHT: " << SPEED_GAP_WEIGHT << endl;
    // cerr << "PATH_PROB_WEIGHT: " << PATH_PROB_WEIGHT << endl;

    // print_input(N, S, M, C, R, v_min, v_max, masses, energies,
    // stones_availability, graph);
    // cerr << "Solving..." << endl;
    // while (1)
    // for(int i=0; i<10000; i++)
    while(1)
        solve(C, pickup_from_city);

    return 0;
}